#!/usr/bin/env python
# -*- coding: utf-8 -*-
# File: atagone.py
#
# Copyright 2018-2025 Wim van den Herik
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
#  of this software and associated documentation files (the "Software"), to
#  deal in the Software without restriction, including without limitation the
#  rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
#  sell copies of the Software, and to permit persons to whom the Software is
#  furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
#  all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#  FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE
#  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
#  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
#  DEALINGS IN THE SOFTWARE.
#

""" a python interface to the ATAG One Thermostat """

from typing import Any, Optional, List
from dataclasses import dataclass, field
from dataclasses_json import dataclass_json, config

@dataclass_json
@dataclass
class AccountAuth:
    user_account: str = ""
    mac_address: str = "01:23:45:67:89:01"
    
@dataclass_json
@dataclass
class Status:
    device_id: Optional[str] = None
    device_status: Optional[int] = None
    connection_status: Optional[int] = None
    date_time: Optional[int] = None

@dataclass_json
@dataclass
class Configuration:
    report_url: Optional[str] = field(default=None, metadata=config(exclude=lambda f: f is None))
    download_url: Optional[str] = field(default=None, metadata=config(exclude=lambda f: f is None))
    boiler_id: Optional[str] = field(default=None, metadata=config(exclude=lambda f: f is None))
    boiler_det_type: Optional[int] = field(default=None, metadata=config(exclude=lambda f: f is None))
    language: Optional[int] = field(default=None, metadata=config(exclude=lambda f: f is None))
    pressure_unit: Optional[int] = field(default=None, metadata=config(exclude=lambda f: f is None))
    temp_unit: Optional[int] = field(default=None, metadata=config(exclude=lambda f: f is None))
    time_format: Optional[int] = field(default=None, metadata=config(exclude=lambda f: f is None))
    time_zone: Optional[int] = field(default=None, metadata=config(exclude=lambda f: f is None))
    summer_eco_mode: Optional[int] = field(default=None, metadata=config(exclude=lambda f: f is None))
    summer_eco_temp: Optional[float] = field(default=None, metadata=config(exclude=lambda f: f is None))
    shower_time_mode: Optional[int] = field(default=None, metadata=config(exclude=lambda f: f is None))
    comfort_settings: Optional[int] = field(default=None, metadata=config(exclude=lambda f: f is None))
    room_temp_offs: Optional[float] = field(default=None, metadata=config(exclude=lambda f: f is None))
    outs_temp_offs: Optional[float] = field(default=None, metadata=config(exclude=lambda f: f is None))
    ch_temp_max: Optional[float] = field(default=None, metadata=config(exclude=lambda f: f is None))
    ch_vacation_temp: Optional[float] = field(default=None, metadata=config(exclude=lambda f: f is None))
    start_vacation: Optional[int] = field(default=None, metadata=config(exclude=lambda f: f is None))
    wd_k_factor: Optional[float] = field(default=None, metadata=config(exclude=lambda f: f is None))
    wd_exponent: Optional[float] = field(default=None, metadata=config(exclude=lambda f: f is None))
    climate_zone: Optional[float] = field(default=None, metadata=config(exclude=lambda f: f is None))
    wd_temp_offs: Optional[float] = field(default=None, metadata=config(exclude=lambda f: f is None))
    dhw_legion_day: Optional[int] = field(default=None, metadata=config(exclude=lambda f: f is None))
    dhw_legion_time: Optional[int] = field(default=None, metadata=config(exclude=lambda f: f is None))
    dhw_boiler_cap: Optional[int] = field(default=None, metadata=config(exclude=lambda f: f is None))
    ch_building_size: Optional[int] = field(default=None, metadata=config(exclude=lambda f: f is None))
    ch_heating_type: Optional[int] = field(default=None, metadata=config(exclude=lambda f: f is None))
    ch_isolation: Optional[int] = field(default=None, metadata=config(exclude=lambda f: f is None))
    installer_id: Optional[str] = field(default=None, metadata=config(exclude=lambda f: f is None))
    disp_brightness: Optional[int] = field(default=None, metadata=config(exclude=lambda f: f is None))
    ch_mode_vacation: Optional[int] = field(default=None, metadata=config(exclude=lambda f: f is None))
    ch_mode_extend: Optional[int] = field(default=None, metadata=config(exclude=lambda f: f is None))
    support_contact: Optional[str] = field(default=None, metadata=config(exclude=lambda f: f is None))
    privacy_mode: Optional[int] = field(default=None, metadata=config(exclude=lambda f: f is None))
    ch_max_set: Optional[float] = field(default=None, metadata=config(exclude=lambda f: f is None))
    ch_min_set: Optional[float] = field(default=None, metadata=config(exclude=lambda f: f is None))
    dhw_max_set: Optional[float] = field(default=None, metadata=config(exclude=lambda f: f is None))
    dhw_min_set: Optional[float] = field(default=None, metadata=config(exclude=lambda f: f is None))
    mu: Optional[float]  = field(default=None, metadata=config(exclude=lambda f: f is None))
    dhw_legion_enabled: Optional[bool] = field(default=None, metadata=config(exclude=lambda f: f is None))
    frost_prot_enabled: Optional[bool] = field(default=None, metadata=config(exclude=lambda f: f is None))
    frost_prot_temp_outs: Optional[float] = field(default=None, metadata=config(exclude=lambda f: f is None))
    frost_prot_temp_room: Optional[float] = field(default=None, metadata=config(exclude=lambda f: f is None))
    wdr_temps_influence: Optional[int] = field(default=None, metadata=config(exclude=lambda f: f is None))
    max_preheat: Optional[int] = field(default=None, metadata=config(exclude=lambda f: f is None))

@dataclass_json
@dataclass
class Control:
    ch_status: Optional[int] = field(default=None, metadata=config(exclude=lambda f: f is None))
    ch_control_mode: Optional[int] = field(default=None, metadata=config(exclude=lambda f: f is None))
    ch_mode: Optional[int] = field(default=None, metadata=config(exclude=lambda f: f is None))
    ch_mode_duration: Optional[int] = field(default=None, metadata=config(exclude=lambda f: f is None))
    ch_mode_temp: Optional[float] = field(default=None, metadata=config(exclude=lambda f: f is None))
    dhw_temp_setp: Optional[float] = field(default=None, metadata=config(exclude=lambda f: f is None))
    dhw_status: Optional[int] = field(default=None, metadata=config(exclude=lambda f: f is None))
    dhw_mode: Optional[int] = field(default=None, metadata=config(exclude=lambda f: f is None))
    dhw_mode_temp: Optional[float] = field(default=None, metadata=config(exclude=lambda f: f is None))
    weather_temp: Optional[float] = field(default=None, metadata=config(exclude=lambda f: f is None))
    weather_status: Optional[int] = field(default=None, metadata=config(exclude=lambda f: f is None))
    vacation_duration: Optional[int] = field(default=None, metadata=config(exclude=lambda f: f is None))
    extend_duration: Optional[int] = field(default=None, metadata=config(exclude=lambda f: f is None))
    fireplace_duration: Optional[int] = field(default=None, metadata=config(exclude=lambda f: f is None))

@dataclass_json
@dataclass
class Details:
    boiler_temp: Optional[float] = None
    boiler_return_temp: Optional[float] = None
    min_mod_level: Optional[int] = None
    rel_mod_level: Optional[int] = None
    boiler_capacity: Optional[int] = None
    target_temp: Optional[float] = None
    overshoot: Optional[float] = None
    max_boiler_temp: Optional[float] = None
    alpha_used: Optional[float] = None
    regulation_state: Optional[int] = None
    ch_m_dot_c: Optional[float] = None
    c_house: Optional[int] = None
    r_rad: Optional[float] = None
    r_env: Optional[float] = None
    alpha: Optional[float] = None
    alpha_max: Optional[float] = None
    delay: Optional[int] = None
    mu: Optional[float] = None
    threshold_offs: Optional[float] = None
    wd_k_factor: Optional[float] = None
    wd_exponent: Optional[float] = None
    lmuc_burner_hours: Optional[int] = None
    lmuc_dhw_hours: Optional[int] = None
    KP: Optional[float] = None
    KI: Optional[float] = None

@dataclass_json
@dataclass
class Report:
    report_time: Optional[int] = None
    burning_hours:Optional[float] = None
    device_errors: Optional[str] = None
    boiler_errors: Optional[str] = None
    room_temp: Optional[float] = None
    outside_temp: Optional[float] = None
    dbg_outside_temp: Optional[float] = None
    pcb_temp: Optional[float] = None
    ch_setpoint: Optional[float] = None
    dhw_water_temp: Optional[float] = None
    ch_water_temp: Optional[float] = None
    dhw_water_pres: Optional[float] = None
    ch_water_pres: Optional[float] = None
    ch_return_temp: Optional[float] = None
    boiler_status: Optional[int] = None
    boiler_config: Optional[int] = None
    ch_time_to_temp: Optional[int] = None
    shown_set_temp: Optional[float] = None
    power_cons: Optional[int] = None
    tout_avg: Optional[float] = None
    rssi: Optional[int] = None
    current: Optional[int] = None
    voltage: Optional[int] = None
    charge_status: Optional[int] = None
    lmuc_burner_starts: Optional[int] = None
    dhw_flow_rate: Optional[float] = None
    resets: Optional[int] = None
    memory_allocation: Optional[int] = None
    details: Optional[Details] = None

@dataclass_json
@dataclass
class Schedule:
    base_temp: Optional[float] = None
    entries: Optional[List[List[List[float]]]] = field(default=None, metadata=config(exclude=lambda f: f is None))

@dataclass_json
@dataclass
class Schedules:
    ch_schedule: Optional[Schedule] = field(default=None, metadata=config(exclude=lambda f: f is None))
    dhw_schedule: Optional[Schedule] = field(default=None, metadata=config(exclude=lambda f: f is None))

@dataclass_json
@dataclass
class UpdateMessage:
    seqnr: int = 0
    account_auth: AccountAuth = None
    configuration: Optional[Configuration]  = field(default=None, metadata=config(exclude=lambda f: f is None))
    schedules: Optional[Schedules] = field(default=None, metadata=config(exclude=lambda f: f is None))
    control: Optional[Control] = field(default=None, metadata=config(exclude=lambda f: f is None))

@dataclass_json
@dataclass
class Update:   
    update_message: UpdateMessage = None

@dataclass_json
@dataclass
class RetrieveReply:
    seqnr: Optional[int] = None
    status: Optional[Status] = None
    report: Optional[Report] = None
    control: Optional[Control] = None
    schedules: Optional[Schedules] = None
    configuration: Optional[Configuration]  = None
    acc_status: Optional[int] = None

@dataclass_json
@dataclass
class AtagRetrieveReply:
    retrieve_reply: RetrieveReply

@dataclass_json
@dataclass
class RetrieveMessage:
    seqnr: int = 0
    account_auth: AccountAuth = None
    info: None = 127
    
@dataclass_json
@dataclass
class Report:   
    retrieve_message: RetrieveMessage = None

@dataclass_json
@dataclass
class Entry:
    user_account: str = ""
    mac_address: str = "01:23:45:67:89:01"
    device_name: str = "Home Assistant"
    account_type: str = 0
    
@dataclass_json
@dataclass
class Accounts:
    entries: Optional[List] = None

@dataclass_json
@dataclass
class PairMessage:
    seqnr: int = 0
    accounts: Accounts = None

@dataclass_json
@dataclass
class Pair:
    pair_message: PairMessage = None

class AtagJson:
    """ Represent the different update messages for the Atag One in Json Format """
    
    def __init__(self) -> None:
        self.update = Update()
        self.retrieve = Report()
        self.pair = Pair()
        
    def ReportJson(self) -> None:
        self.retrieve.retrieve_message = RetrieveMessage()
        self.retrieve.retrieve_message.account_auth = AccountAuth()
        return self.retrieve.to_json(sort_keys=False)
    
    def PairJson(self) -> None:
        self.pair.pair_message = PairMessage()
        self.pair.pair_message.accounts = Accounts(entries=[Entry()])
        return self.pair.to_json(sort_keys=False)
    
    def _UpdateJson(self) -> None:
        self.update.update_message = UpdateMessage()
        self.update.update_message.account_auth = AccountAuth()
    
    """support to call functions dynamically """
    def update_for(self, name: str, *args, **kwargs):
        updatefunction = f"{name}_json"
        if hasattr(self, updatefunction) and callable(func := getattr(self, updatefunction)):
            return func(*args, **kwargs)
    
    """ HVAC Mode"""
    def ch_control_mode_json(self, hvac_mode: int) -> None:
        self._UpdateJson()
        self.update.update_message.control=Control(ch_control_mode=hvac_mode)
        return self.update.to_json(sort_keys=False)

    """ Preset Mode """
    def ch_mode_json(self, preset_mode: int) -> None:
        self._UpdateJson()
        self.update.update_message.control=Control(ch_mode=preset_mode)
        return self.update.to_json(sort_keys=False)
    
    def ch_mode_temp_json(self, target_temp: float) -> None:
        self._UpdateJson()
        self.update.update_message.control=Control(ch_mode_temp=target_temp)
        return self.update.to_json(sort_keys=False)    
    
    def dhw_temp_setp_json(self, target_temp: float) -> None:
        self._UpdateJson()
        self.update.update_message.control=Control(dhw_temp_setp=target_temp)
        return self.update.to_json(sort_keys=False)
    
    def dhw_schedule_json(self, schedule: Schedule) -> None:
        self._UpdateJson()
        self.update.update_message.schedules = Schedules(dhw_schedule=schedule)
        return self.update.to_json(sort_keys=False)
    
    def ch_schedule_json(self, schedule: Schedule) -> None:
        self._UpdateJson()
        self.update.update_message.schedules = Schedules(ch_schedule=schedule)
        return self.update.to_json(sort_keys=False)

    def dhw_mode_json(self, mode: int) -> None:
        self._UpdateJson()
        self.update.update_message.control=Control(dhw_mode=mode)
        return self.update.to_json(sort_keys=False)
    
    """ Set Configuration """
    def create_vacation_json(self, start_dt_epoch: int, heat_temp: float, duration: int ) -> None:
        self._UpdateJson()
        self.update.update_message.configuration = Configuration(start_vacation=start_dt_epoch, ch_vacation_temp=heat_temp)
        self.update.update_message.control = Control(vacation_duration=duration)
        return self.update.to_json(sort_keys=False)
    
    def cancel_vacation_json(self) -> None:
        self._UpdateJson()
        self.update.update_message.configuration = Configuration(start_vacation=0)
        self.update.update_message.control = Control(vacation_duration=0, ch_mode=2)
        return self.update.to_json(sort_keys=False)
    
    def outs_temp_offs_json(self, correction: float) -> None:
        self._UpdateJson()
        self.update.update_message.configuration = Configuration(outs_temp_offs=correction)
        return self.update.to_json(sort_keys=False)

    def room_temp_offs_json(self, correction: float) -> None:
        self._UpdateJson()
        self.update.update_message.configuration = Configuration(room_temp_offs=correction)
        return self.update.to_json(sort_keys=False)
    
    def summer_eco_mode_json(self, mode: int) -> None:
        self._UpdateJson()
        self.update.update_message.configuration = Configuration(summer_eco_mode=mode)
        return self.update.to_json(sort_keys=False)
    
    def summer_eco_temp_json(self, eco_temp: float) -> None:
        self._UpdateJson()
        self.update.update_message.configuration = Configuration(summer_eco_temp=eco_temp)
        return self.update.to_json(sort_keys=False)
    
    def ch_vacation_temp_json(self, heat_temp: float) -> None:
        self._UpdateJson()
        self.update.update_message.configuration = Configuration(ch_vacation_temp=heat_temp)
        return self.update.to_json(sort_keys=False)
      
    def ch_building_size_json(self, building_size: int) -> None:
        self._UpdateJson()
        self.update.update_message.configuration = Configuration(ch_building_size=building_size)
        return self.update.to_json(sort_keys=False)
    
    def ch_isolation_json(self, isolation: int) -> None:
        self._UpdateJson()
        self.update.update_message.configuration = Configuration(ch_isolation=isolation)
        
    def ch_heating_type_json(self, heating_type: int) -> None:
        self._UpdateJson()
        self.update.update_message.configuration = Configuration(ch_heating_type=heating_type)
        return self.update.to_json(sort_keys=False)
    
    def wdr_temps_influence_json(self, influence: int) -> None:
        self._UpdateJson()
        self.update.update_message.configuration = Configuration(wdr_temps_influence=influence)
        return self.update.to_json(sort_keys=False)
    
    def frost_prot_enabled_json(self, enabled: int) -> None:
        self._UpdateJson()
        self.update.update_message.configuration = Configuration(frost_prot_enabled=enabled)
        return self.update.to_json(sort_keys=False)
    
    


#!/usr/bin/env python
# # -*- coding: utf-8 -*-
# # File: exceptions.py
# #
# # Copyright 2018 Wim van den Herik
# #
# # Permission is hereby granted, free of charge, to any person obtaining a copy
# #  of this software and associated documentation files (the "Software"), to
# #  deal in the Software without restriction, including without limitation the
# #  rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
# #  sell copies of the Software, and to permit persons to whom the Software is
# #  furnished to do so, subject to the following conditions:
# #
# # The above copyright notice and this permission notice shall be included in
# #  all copies or substantial portions of the Software.
# #
# # THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# #  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# #  FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE
# #  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# #  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# #  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# #  DEALINGS IN THE SOFTWARE.
#
""" Helper objects for pyatagone """

__author__ = '''Wim van den Herik'''
__docFormat__ = 'plaintext'
__date__ = '''27-09-2025'''
__copyright__ = '''Copyright 2025, Wim van den Herik'''
__credits__ = ["Wim van den Herik"]
__license__ = '''MIT'''
__maintainer__ = '''Wim van den Herik'''
__email__ = '''<wvdh2002@gmail.com>'''
__status__ = '''Development'''  # "Prototype", "Development", "Production".


class AtagNotPaired(Exception):
    """ Calling client not paired with Atag One """

class AtagInvalidResponse(Exception):
    """ Invalid response is received from Atag One """
    
class AtagStatusException(Exception):
    """ Status Exception for status other then 2 """
    
class AtagConnectException(Exception):
    """ Atag Connection Exception """



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# File: atagone.py
#
# Copyright 2018-2025 Wim van den Herik
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
#  of this software and associated documentation files (the "Software"), to
#  deal in the Software without restriction, including without limitation the
#  rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
#  sell copies of the Software, and to permit persons to whom the Software is
#  furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
#  all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#  FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE
#  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
#  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
#  DEALINGS IN THE SOFTWARE.
#

""" a python interface to the ATAG One Thermostat """

import logging
from enum import Enum, StrEnum
from typing import Dict, Final


_LOGGER = logging.getLogger(__package__)

HEADERS: Final[Dict[str, str]] = {
'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/119.0.0.0 Safari/537.36',
'X-OneApp-Version': 'R34 (18781)',
'Content-Type': 'application/x-www-form-urlencoded;charset=UTF-8',
}

DEFAULT_TIMEOUT = 30
DEFAULT_MIN_TEMP = 4
DEFAULT_MAX_TEMP = 27
DEFAULT_PORT = 10000




#!/usr/bin/env python
# -*- coding: utf-8 -*-
# File: atagone.py
#
# Copyright 2018-2025 Wim van den Herik
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
#  of this software and associated documentation files (the "Software"), to
#  deal in the Software without restriction, including without limitation the
#  rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
#  sell copies of the Software, and to permit persons to whom the Software is
#  furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
#  all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#  FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE
#  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
#  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
#  DEALINGS IN THE SOFTWARE.
#

""" a python interface to the ATAG One Thermostat """

from datetime import datetime, timedelta
from typing import Any, Dict, Optional
import aiohttp
import asyncio
import atexit
import logging
from http import HTTPStatus
from .atagoneentity import AtagOneEntity
from .exceptions import AtagConnectException, AtagStatusException

from socket import AF_INET, SOCK_DGRAM, SO_REUSEADDR, SOL_SOCKET, socket, timeout

from .atagonejson import AtagJson


BASE_URL = "http://{0}:{1}{2}"
READ_PATH = "/retrieve"
UPDATE_PATH = "/update"
PAIR_PATH = "/pair_message"
MESSAGE_INFO_CONTROL = 1
MESSAGE_INFO_SCHEDULES = 2
MESSAGE_INFO_CONFIGURATION = 4
MESSAGE_INFO_REPORT = 8
MESSAGE_INFO_STATUS = 16
MESSAGE_INFO_WIFISCAN = 32  
MESSAGE_INFO_EXTRA = 64
MESSAGE_INFO_REPORT_DETAILS = 64

_LOGGER = logging.getLogger("atagoneapi")
_LOGGER.setLevel(logging.DEBUG)

    
class AtagDiscovery(asyncio.DatagramProtocol):
    """Atag Datagram Protocol Discovery class """
    
    def connection_made(self, transport):
        self.transport = transport
        self.data = asyncio.Future()

    def datagram_received(self, data, addr):
        if not self.data.done():
            self.data.set_result([data, addr])

class AtagOneApi(AtagOneEntity):
    """Wrapper class to the Atag One Local API"""

    def __init__(self, host: Optional[str] = None, port: Optional[int] = 10000):
        self.data = None
        self.paired = False
        self.heating = False
        self.port = port
        self.host = host
        self.client = None

        self._session: Optional[aiohttp.ClientSession] = None
        self._session_timeout = aiohttp.ClientTimeout(total=15)
        atexit.register(self._close)

    async def initialize(self):
        """Initialize the API, performing discovery if needed."""
        if not self.host:
            _LOGGER.info("No host provided. Starting discovery...")
            self.host = await self.async_discover()

            if not self.host:
                _LOGGER.error("Failed to discover ATAG One thermostat on the network.")
                raise ConnectionError("Could not discover ATAG One thermostat")

        _LOGGER.debug(f"Using ATAG One at host {self.host}:{self.port}")

        # Lazy session creation
        if not self._session:
            self._session = aiohttp.ClientSession(timeout=self._session_timeout)

    async def async_discover(self) -> Optional[str]:
        """Discover the Atag One thermostat on the local network."""
        loop = asyncio.get_running_loop()
        addr = None

        _LOGGER.debug("Starting ATAG One discovery on UDP port 11000...")

        try:
            transport, protocol = await loop.create_datagram_endpoint(
                lambda: AtagDiscovery(),
                local_addr=("0.0.0.0", 11000),
            )
            try:
                _, (addr, _) = await asyncio.wait_for(protocol.data, timeout=30)
                _LOGGER.info(f"Discovered ATAG One at {addr}")
            except asyncio.TimeoutError:
                _LOGGER.warning("Discovery timed out — no ATAG One detected")
            finally:
                transport.close()
        except OSError as e:
            _LOGGER.error(f"Failed to open UDP socket for discovery: {e}")

        return addr
    
    async def async_create_vacation(
        self, 
        start_date: Optional[int] = None, 
        start_time: Optional[int] = None, 
        end_date: Optional[int]  = None, 
        end_time: Optional[int] = None, 
        heat_temp: Optional[float] = None ) -> bool:
        """create vacation on the Atag One"""

        if not any([start_date, start_time, end_date, end_time]):
            """ if no startdate/enddate/time specified, just set 14 days from now """
            start_dt_epoch = self._datetime_atag(
                datetime.now().strftime("%Y-%m-%d %H:%M:%S")
            )
            end_dt_epoch = self._datetime_atag(
                (datetime.now() + timedelta(days=14)).strftime("%Y-%m-%d %H:%M:%S")
            )
        else:
            if not all([start_date, start_time, end_date, end_time]):
                raise ValueError("Start and end date/time must be provided together")
            start_dt_epoch = self._datetime_atag(f"{start_date} {start_time}")
            end_dt_epoch = self._datetime_atag(f"{end_date} {end_time}")

        heat_temp = float(heat_temp) if heat_temp is not None else 20.0
        duration = end_dt_epoch - start_dt_epoch

        json_payload = AtagJson().create_vacation_json(start_dt_epoch, heat_temp, duration)
        if json_payload is not None:
            response = await self._async_send_request(UPDATE_PATH, json_payload)
            if response:
                return True

        return False

    async def async_cancel_vacation(self) -> bool:
        """cancel vacation on the Atag One"""
                
        json_payload = AtagJson().cancel_vacation_json()
        response = await self._async_send_request(UPDATE_PATH, json_payload)
        if response:
            return True

        return False
    
    async def send_dynamic_change(self, field_to_update, value) -> bool:

        jsonpayload = AtagJson().update_for(field_to_update, value)
        if jsonpayload:
            response = await self._async_send_request(UPDATE_PATH, jsonpayload)
            if response:
                return True

        return False
    
    async def async_dhw_schedule_base_temp(self, value) -> bool:
        """Set the DHW temperature setpoint"""

        dhw_schedule = self.dhwscheduledata
        if not dhw_schedule:
            raise ValueError("No DHW schedule data available; call async_update first")
        dhw_schedule["base_temp"] = value
        jsonpayload = AtagJson().dhw_schedule_json(dhw_schedule)
        if jsonpayload:
            response = await self._async_send_request(UPDATE_PATH, jsonpayload) 
            if response:
                return True
             
        return False
    
    async def async_ch_schedule_base_temp(self, value) -> bool:
        """Set the CH temperature setpoint"""

        ch_schedule = self.chscheduledata
        if not ch_schedule:
            raise ValueError("No CH schedule data available; call async_update first")
        ch_schedule["base_temp"] = value
        jsonpayload = AtagJson().ch_schedule_json(ch_schedule)
        if jsonpayload:
            response = await self._async_send_request(UPDATE_PATH, jsonpayload) 
            if response:
                return True
             
        return False

    async def async_fetch_data(self) -> dict:
        """Get state of all sensors and do some conversions"""

        await self.async_update()
        return self.sensors

    async def _ensure_session(self) -> aiohttp.ClientSession:
        if self._session is None or self._session.closed:
            self._session = aiohttp.ClientSession(timeout=self._session_timeout)
        return self._session

    async def async_update(self) -> bool:
        """Report Data"""

        json_payload = AtagJson().ReportJson()
        resp = await self._async_send_request(READ_PATH, json_payload)
        if not resp:
            return False

        self.data = resp["retrieve_reply"]
        
        boiler_status = self._coerce_int(self.reportdata.get("boiler_status")) or 0
        status = boiler_status & 14
        self.heating = status == 10

        return True
    
    async def _async_send_request(self, request_path, json_payload, is_retry: bool = False ) -> Optional[Dict[str, Any]]:
        """send async web request"""

        if not self.host:
            raise AtagConnectException(
                "Host is unknown; set 'host' or call async_discover() before sending requests"
            )

        session = await self._ensure_session()
        url = BASE_URL.format(self.host, self.port, request_path)

        try:
            async with session.post(url, data=json_payload.encode()) as response:
                if response.status == HTTPStatus.NOT_FOUND:
                    raise AtagConnectException("404: page not found")

                if response.status >= HTTPStatus.BAD_REQUEST:
                    if not is_retry:
                        await asyncio.sleep(5)
                        return await self._async_send_request(request_path, json_payload, True)
                    raise AtagConnectException(f"Error {response.status} calling {url}")

                if response.content_length and response.content_length > 0:
                    payload = await response.json()
                else:
                    payload = await response.json(content_type=None)

                if self.__check_response(request_path, payload):
                    return payload
        except asyncio.TimeoutError as exc:
            raise AtagConnectException("Timeout while communicating with ATAG One") from exc
        except aiohttp.ClientError as exc:
            if not is_retry:
                await asyncio.sleep(5)
                return await self._async_send_request(request_path, json_payload, True)
            raise AtagConnectException("Unable to communicate with ATAG One") from exc

        return None
    
    def __check_response(self, request_path, response: Dict[str, Any]) -> bool:
        if request_path == UPDATE_PATH:
            status = response["update_reply"]["acc_status"]
            if status != 2:
                raise AtagStatusException("Error: Can't update Atag One", status)
        elif request_path == READ_PATH:
            status = response["retrieve_reply"]["acc_status"]
            if status != 2:
                raise AtagStatusException("Error: Can't read Atag One", status)

        return True
    

    async def async_pair_atag(self) -> None:
        """Pair the Thermostat"""

        json_payload = AtagJson().PairJson()
        resp = await self._async_send_request(PAIR_PATH, json_payload)
        if resp is None:
            raise AtagConnectException("No response received when pairing with ATAG One")

        data = resp["pair_reply"]
        status = data["acc_status"]
        if status == 2:
            self.paired = True
            return
        elif status == 1:
            self.paired = True
            _LOGGER.error("Waiting for pairing confirmation")
        elif status == 3:
            _LOGGER.error("Waiting for pairing confirmation")
        elif status == 0:
            _LOGGER.error("No status returned from ATAG One")

        self.paired = False

    def _close(self) -> None:
        if self._session and not self._session.closed:
            _LOGGER.debug("closing connection")
            try:
                loop = asyncio.get_running_loop()
            except RuntimeError:
                asyncio.run(self._session.close())
                return

            async def _close_session():
                if self._session and not self._session.closed:
                    await self._session.close()

            if loop.is_running():
                asyncio.run_coroutine_threadsafe(_close_session(), loop)
            else:
                loop.run_until_complete(_close_session())



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# File: atagone.py
#
# Copyright 2018-2025 Wim van den Herik
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
#  of this software and associated documentation files (the "Software"), to
#  deal in the Software without restriction, including without limitation the
#  rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
#  sell copies of the Software, and to permit persons to whom the Software is
#  furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
#  all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#  FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE
#  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
#  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
#  DEALINGS IN THE SOFTWARE.
#
"""High level client for interacting with the ATAG One thermostat."""

from __future__ import annotations

from typing import Optional, Union

from .atagoneapi import AtagOneApi
from .const import DEFAULT_MAX_TEMP, DEFAULT_MIN_TEMP, DEFAULT_PORT
from .atagonejson import Schedule


class AtagOne(AtagOneApi):
    """High-level Atag One client that exposes user-friendly helpers."""

    def __init__(self, host: Optional[str] = None, port: int = DEFAULT_PORT) -> None:
        super().__init__(host=host, port=port)

    async def async_discover(self) -> Optional[str]:
        """Discover the thermostat and remember the host that answered."""
        addr = await super().async_discover()
        if addr:
            self.host = addr
        return addr

    async def async_ch_mode(self, mode: int) -> bool:
        """Set the CH preset mode (manual/auto/etc.)."""
        return await self.send_dynamic_change("ch_mode", mode)

    async def async_set_temperature(self, temperature: Union[int, float]) -> bool:
        """Set new target temperature."""

        target = float(temperature)
        if target < DEFAULT_MIN_TEMP or target > DEFAULT_MAX_TEMP:
            raise ValueError(
                f"Target temperature must be between {DEFAULT_MIN_TEMP} and {DEFAULT_MAX_TEMP}"
            )
        return await self.send_dynamic_change("ch_mode_temp", target)

    async def async_ch_control_mode(self, hvac_mode: int) -> bool:
        """Set the CH control mode (on/off)."""
        return await self.send_dynamic_change("ch_mode", hvac_mode)
          
    async def async_dhw_temp_setp(self, target_temp: float) -> bool:
        """Set new DHW target temperature."""
        return await self.send_dynamic_change("dhw_mode_temp", target_temp)
    
    async def async_dhw_schedule(self, schedule: Schedule) -> bool:
        """Set new DHW schedule."""
        return await self.send_dynamic_change("dhw_schedule", schedule)
    
    async def async_ch_schedule(self, schedule: Schedule) -> bool:
        """Set new CH schedule."""
        return await self.send_dynamic_change("ch_schedule", schedule)

    async def async_dhw_mode(self, mode: int) -> bool:
        """Set the DHW preset mode (manual/auto/etc.)."""
        return await self.send_dynamic_change("dhw_mode", mode)
    
    async def async_create_vacation(self, start_dt_epoch: int, heat_temp: float, duration: int ) -> bool:
        """Create a vacation."""
        return await self.send_dynamic_change(
            "create_vacation",
            {
                "start_vacation": start_dt_epoch,
                "ch_vacation_temp": heat_temp,
                "vacation_duration": duration,
            },
        ) 

    async def async_cancel_vacation(self) -> bool:
        """Cancel the current vacation."""
        return await self.send_dynamic_change("cancel_vacation", {})
    
    async def async_outs_temp_offs(self, correction: float) -> bool:
        """Set the outside temperature offset correction."""
        return await self.send_dynamic_change("outs_temp_offs", correction)

    async def async_room_temp_offs(self, correction: float) -> bool:
        """Set the room temperature offset correction."""
        return await self.send_dynamic_change("room_temp_offs", correction)
    
    async def async_summer_eco_mode(self, mode: int) -> bool:
        """Set the summer eco mode (on/off)."""
        return await self.send_dynamic_change("summer_eco_mode", mode)
    
    async def async_summer_eco_temp(self, eco_temp: float) -> bool:
        """Set the summer eco temperature."""
        return await self.send_dynamic_change("summer_eco_temp", eco_temp)
    
    async def async_ch_vacation_temp(self, heat_temp: float) -> bool:
        """Set the CH vacation temperature."""
        return await self.send_dynamic_change("ch_vacation_temp", heat_temp)
      
    async def async_ch_building_size(self, building_size: int) -> bool:
        """Set the CH building size."""
        return await self.send_dynamic_change("ch_building_size", building_size)
    
    async def async_ch_isolation(self, isolation: int) -> bool:
        """Set the CH isolation."""
        return await self.send_dynamic_change("ch_isolation", isolation)
        
    async def async_ch_heating_type(self, heating_type: int) -> bool:
        """Set the CH heating type."""
        return await self.send_dynamic_change("ch_heating_type", heating_type)
    
    async def async_wdr_temps_influence(self, influence: int) -> bool:
        """Set the WDR temps influence."""
        return await self.send_dynamic_change("wdr_temps_influence", influence) 
    
    async def async_frost_prot_enabled(self, enabled: int) -> bool:
        """Set the frost protection enabled."""
        return await self.send_dynamic_change("frost_prot_enabled", enabled)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# File: atagone.py
#
# Copyright 2018-2025 Wim van den Herik
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
#  of this software and associated documentation files (the "Software"), to
#  deal in the Software without restriction, including without limitation the
#  rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
#  sell copies of the Software, and to permit persons to whom the Software is
#  furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
#  all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#  FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE
#  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
#  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
#  DEALINGS IN THE SOFTWARE.
#

""" a python interface to the ATAG One Thermostat """


from datetime import datetime, timedelta
import logging
from typing import Any, Dict, Optional, Union
from urllib.parse import urlparse


_LOGGER = logging.getLogger(__name__)

ATAG_EPOCH = datetime(2000, 1, 1)


class AtagOneEntity(object):
    """Base entity for the Atag ONE API wrappers."""

    def __init__(self):
        self.data: Optional[Dict[str, Any]] = None
        self.heating: bool = False

    def _get_section(self, key: str) -> Dict[str, Any]:
        """Return a section from the data payload, defaulting to an empty dict."""
        if isinstance(self.data, dict):
            section = self.data.get(key, {})
            if isinstance(section, dict):
                return section
        return {}

    @property
    def id(self) -> Optional[str]:
        """Return the ID of the Atag One."""
        status = self._get_section("status")
        return status.get("device_id")

    @property
    def reportdata(self) -> Dict[str, Any]:
        """Return Report Json Data"""
        return self._get_section("report")

    @property
    def controldata(self) -> Dict[str, Any]:
        """Return Control Json Data"""
        return self._get_section("control")

    @property
    def scheduledata(self) -> Dict[str, Any]:
        """Return Schedules Json Data"""
        return self._get_section("schedules")
    
    @property
    def chscheduledata(self) -> Dict[str, Any]:
        """Return CH schedules JSON data."""
        schedule = self.scheduledata.get("ch_schedule")
        return schedule if isinstance(schedule, dict) else {}
    
    @property
    def dhwscheduledata(self) -> Dict[str, Any]:
        """Return DHW schedules JSON data."""
        schedule = self.scheduledata.get("dhw_schedule")
        return schedule if isinstance(schedule, dict) else {}
    
    @property
    def configurationdata(self) -> Dict[str, Any]:
        """Return Configuration Json Data"""
        return self._get_section("configuration")

    @property
    def current_setpoint(self) -> Optional[float]:
        """Return current setpoint temp"""
        return self.reportdata.get("shown_set_temp")

    @property
    def current_temp(self) -> float:
        """Return current temp"""
        return self.reportdata.get("room_temp", 0)

    @property
    def mode(self) -> int:
        return self.controldata.get("ch_control_mode", 0)
    
    @property
    def vacation_duration(self) -> int:
        return self.controldata.get("vacation_duration", 0)

    @property
    def preset(self) -> int:
        return self.controldata.get("ch_mode", 2)
    
    @property
    def firmware_version(self) -> Optional[str]:
        download_url = self.configurationdata.get("download_url")
        if not download_url:
            return None

        return urlparse(download_url).path.replace("/", "") or None
    
    @property
    def sensors(self) -> Dict[str, Any]:
        """Get all sensors from the report data"""
        sensors = {}
        report = self.reportdata
        details = report.get("details")
        if not isinstance(details, dict):
            details = {}

        for sensor, value in report.items():
            if sensor == "details":
                continue
            if sensor == "tout_avg":
                sensors[sensor] = value
                sensors["avg_outside_temp"] = value
                continue
            sensors[sensor] = value

        for sensor, value in details.items():
            sensors[sensor] = value

        for sensor, value in self.controldata.items():
            sensors[sensor] = value

        sensors["voltage"] = self.voltage
        sensors["power_cons"] = self.power_cons
        sensors["rssi"] = self.rssi

        sensors["summer_eco_temp"] = self.configurationdata.get("summer_eco_temp")

        return sensors
    
    @property
    def voltage(self) -> float:
        """convert Voltage mV into V"""
        voltage = self._coerce_int(self.reportdata.get("voltage"))
        if voltage is None:
            return 0

        if voltage > 1000:
            return voltage / 1000

        return voltage
    
    @property
    def power_cons(self) -> float:
        """convert power_cons to m3/h """
        power_cons = self._coerce_int(self.reportdata.get("power_cons"))
        if power_cons is None:
            return 0

        if power_cons > 0:
            return power_cons / 100000
        return 0
    
    @property
    def rssi(self) -> int:
        """ convert to dBm """
        rssi_value = self._coerce_int(self.reportdata.get("rssi"))
        if rssi_value is None:
            return 0

        return -rssi_value

    @staticmethod
    def _coerce_int(value: Any) -> Optional[int]:
        """Convert a value to int, returning None when conversion fails."""
        try:
            return int(value)
        except (TypeError, ValueError):
            return None
    
    def _atag_datetime(self, localtime: Union[int, float]) -> datetime:
        """Convert ATAG datetime (seconds since 2000 epoch) to datetime object."""
        return ATAG_EPOCH + timedelta(seconds=localtime)

    def _datetime_atag(self, dtstring: Union[str, datetime]) -> int:
        """Convert datetime (or formatted string) to ATAG epoch seconds."""
        if isinstance(dtstring, datetime):
            dt_value = dtstring
        else:
            dt_value = datetime.strptime(str(dtstring), "%Y-%m-%d %H:%M:%S")

        return int((dt_value - ATAG_EPOCH).total_seconds())
        
